@@ -8,9 +8,7 @@ extern crate failure;
 extern crate futures;
 extern crate log;
 extern crate minidom;
-extern crate quick_xml;
 extern crate reqwest;
 extern crate serde;
-extern crate serde_derive;
 extern crate serde_json;
-extern crate tokio_core;
+extern crate tokio;
